<?php

namespace Drupal\simple_survey\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Url;
use Drupal\simple_survey\Entity\SimpleSurveyInterface;

/**
 * Class SimpleSurveyController.
 *
 * @package Drupal\simple_survey\Controller
 */
class SimpleSurveyController extends ControllerBase {
  use MessengerTrait;

  /**
   * The modules machine name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Drupal date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The Drupal Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * File System service.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * Simple Survery Controller constructor.
   *
   * @todo Look into creating a service to handle DI. Doing a janky call in feedbackjs.module
   *
   * @param Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The Drupal date formatter service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The Drupal time service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystemInterface
   *   The Drupal file system service.
   */
  public function __construct(
    DateFormatter $dateFormatter,
    TimeInterface $time,
    FileSystemInterface $fileSystemInterface) {
    $this->dateFormatter = $dateFormatter;
    $this->time = $time;
    $this->fileSystem = $fileSystemInterface;

    $this->modulePath = drupal_get_path('module', $this->moduleName);
  }

  /**
   * Dependency Injection.
   *
   * @inheritDoc
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('datetime.time'),
      $container->get('file_system')
    );
  }

  /**
   * Allows user to download the temporary generated response survey.
   *
   * @param string $simple_survey
   *   The survey machine name.
   * @param string $filename
   *   The survey filename.
   *
   * @return mixed
   *   Will return either a file or redirect to the requesting survey.
   */
  public function getTemporaryFile($simple_survey, $filename) {

    $filepath = $this->fileSystem->realpath("temporary://")
      . '/' . $filename;

    if (file_exists($filepath)) {
      $file = new \SplFileObject($filepath, 'r');

      $headerResponse = new Response();
      $headerResponse->headers->set('Content-Type', 'test/csv; charset=utf-8');
      $headerResponse->headers->set('Content-Disposition', 'attachment; filename="'
      . $filename);
      $headerResponse->headers->set('Content-Length', $file->getSize());
      $headerResponse->sendHeaders();

      $file->fpassthru();

      return $headerResponse;
    }
    else {
      $this->messenger()->addWarning($this->t(':filename no longer exist. Please generate a new download.', [
        ':filename' => $filename,
      ]));

      return $this->redirect('entity.simple_survey.edit_form', [
        'simple_survey' => $simple_survey,
      ]);
    }
  }

  /**
   * Export responses to CSV.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Returns response object.
   */
  public function exportCsv(SimpleSurveyInterface $simple_survey) {
    $fileTime = $this->dateFormatter->format($this->time->getRequestTime(), 'custom', 'YdmHis');
    $filename = $simple_survey->id() . '-survey-export-' . $fileTime . '.csv';
    $survey = $simple_survey->id();

    $batch = [
      'title' => $this->t('Creating CSV export :filename for :survey', [
        ':survey' => $simple_survey->label(),
        ':filename' => $filename,
      ]),
      'operations' => [
        [
          '\Drupal\simple_survey\Service\ResponseExportService::processData',
          [
            $filename,
            $survey,
          ],
        ],
      ],
      'finished' => '\Drupal\simple_survey\Controller\ResponseExportService::finishedFileProcessing',
      'progress_message' => $this->t('Time left: @estimate | Time elapsed: @elapsed', [
        '@source' => $simple_survey->label(),
      ]),
      'error_message' => $this->t('The export progress has encountered an error.'),
    ];

    batch_set($batch);

    return batch_process(Url::fromRoute('entity.simple_survey.edit_form', [
      'simple_survey' => $survey,
    ]));
  }

  /**
   * Displays information about a survey.
   *
   * @param \Drupal\simple_survey\Entity\SimpleSurveyInterface $simple_survey
   *   The survey to display.
   *
   * @return array
   *   An array suitable for drupal_render().
   */
  public function page(SimpleSurveyInterface $simple_survey) {
    /** @var \Drupal\simple_survey\Service\SurveyState $state */
    $state = \Drupal::service('simple_survey.survey_state');
    $lastExported = $state->getLastExported($simple_survey->id());

    // Build the search index information.
    $render = [
      'view' => [
        '#theme' => 'simple_survey',
        '#index' => $simple_survey,
        '#questionsPath' => $simple_survey->get('questionsPath'),
        '#submitPath' => $simple_survey->get('submitPath'),
        '#lastExported' => $lastExported,
      ],
    ];

    return $render;
  }

  /**
   * Returns the page title for an survey's "View" tab.
   *
   * @param \Drupal\simple_survey\Entity\SimpleSurveyInterface $simple_survey
   *   The survey that is displayed.
   *
   * @return string
   *   The page title.
   */
  public function pageTitle(SimpleSurveyInterface $simple_survey) {
    return new FormattableMarkup('@title', ['@title' => $simple_survey->label()]);
  }

}
